#include "globus_common.h"
#include "globus_error_rollback_contact.h"
#include <string.h>

/**
 * Allocate and initialize an error of type GLOBUS_ERROR_TYPE_ROLLBACK_CONTACT
 *
 * @param base_source
 * @param base_cause
 * @param fmt
 * @param ...
 */ 
extern
globus_object_t *
globus_error_construct_rollback_contact(
    globus_module_descriptor_t *        base_source,
    globus_object_t *                   base_cause,
    const char *                        fmt,
    ...)
{
    globus_object_t *                   error;
    globus_object_t *                   newerror;
    va_list                             ap;

    va_start(ap, fmt);

    newerror = globus_object_construct(GLOBUS_ERROR_TYPE_ROLLBACK_CONTACT);
    error = globus_error_initialize_rollback_contact(
        newerror,
        base_source,
        base_cause,
        fmt,
        ap);

    va_end(ap);
    if (error == NULL)
    {
        globus_object_free(newerror);
    }

    return error;
}
/* globus_error_construct_rollback_contact() */

/* initialize and return an error of type
 * GLOBUS_ERROR_TYPE_ROLLBACK_CONTACT
 */
extern globus_object_t *
globus_error_initialize_rollback_contact(
    globus_object_t *                   error,
    globus_module_descriptor_t *        base_source,
    globus_object_t *                   base_cause,
    const char *                        fmt,
    va_list                             ap)
{
    char *                              instance_data;
    static FILE *                       f = NULL;
    int                                 len;
    va_list                             ap_copy;

    globus_libc_lock();
    if(f == NULL)
    {
        f = fopen("/dev/null", "w");
    }

    globus_libc_va_copy(ap_copy,ap);
    
    len = vfprintf(f, fmt, ap_copy) + 1;

    va_end(ap_copy);
    
    instance_data = malloc(len);

    vsprintf(instance_data, fmt, ap);

    globus_libc_unlock();

    globus_object_set_local_instance_data(error, instance_data);

    return globus_error_initialize_base(error,
                                        base_source,
                                        base_cause);
}

static
void
globus_l_error_rollback_contact_copy(
    void *                              src,
    void **                             dst)
{
    if(src == NULL || dst == NULL) return;
    (*dst) = (void *) globus_libc_strdup((char *)src);
}

static
void
globus_l_error_rollback_contact_free(
    void *                              data)
{
    globus_libc_free(data);
}

static
char *
globus_l_error_rollback_contact_printable(
    globus_object_t *                   error)
{
    return globus_libc_strdup(globus_object_get_local_instance_data(error));
}

const globus_object_type_t GLOBUS_ERROR_TYPE_ROLLBACK_CONTACT_DEFINITION
= globus_error_type_static_initializer (
    GLOBUS_ERROR_TYPE_BASE,
    globus_l_error_rollback_contact_copy,
    globus_l_error_rollback_contact_free,
    globus_l_error_rollback_contact_printable);




